@@ -139,5 +139,12 @@ var LuigiAPI = (function() {
         jsonRPC(this.urlRoot + "/disable_worker", {'worker': workerId});
     }
 
+    LuigiAPI.prototype.setWorkerProcesses = function(workerId, n, callback) {
+        var data = {worker: workerId, n: n};
+        jsonRPC(this.urlRoot + "/set_worker_processes", data, function(response) {
+            callback();
+        });
+    }
+
     return LuigiAPI;
 })();
